@@ -76,6 +76,20 @@ typedef struct _ClutterMainContext      ClutterMainContext;
 #define CLUTTER_ACTOR_IN_RELAYOUT(a)            ((CLUTTER_PRIVATE_FLAGS (a) & CLUTTER_IN_RELAYOUT) != FALSE)
 #define CLUTTER_STAGE_IN_RESIZE(a)              ((CLUTTER_PRIVATE_FLAGS (a) & CLUTTER_IN_RESIZE) != FALSE)
 
+#define CLUTTER_PARAM_READABLE  (G_PARAM_READABLE | G_PARAM_STATIC_STRINGS)
+#define CLUTTER_PARAM_WRITABLE  (G_PARAM_WRITABLE | G_PARAM_STATIC_STRINGS)
+#define CLUTTER_PARAM_READWRITE (G_PARAM_READABLE | G_PARAM_WRITABLE | G_PARAM_STATIC_STRINGS)
+
+/* automagic interning of a static string */
+#define I_(str)  (g_intern_static_string ((str)))
+
+/* mark all properties under the "Property" context */
+#ifdef ENABLE_NLS
+#define P_(String) (_clutter_gettext ((String)))
+#else
+#define P_(String) (String)
+#endif
+
 typedef enum {
   CLUTTER_ACTOR_UNUSED_FLAG = 0,
 
@@ -127,7 +141,7 @@ struct _ClutterMainContext
   ClutterActor *pointer_grab_actor;
   ClutterActor *keyboard_grab_actor;
 
-  /* stack of overridden shaders during paint */
+  /* stack of actors with shaders during paint */
   GSList *shaders;
 
   /* fb bit masks for col<->id mapping in picking */
@@ -144,8 +158,6 @@ struct _ClutterMainContext
   ClutterEvent *current_event;
   guint32 last_event_time;
 
-  gulong redraw_count;
-
   /* list of repaint functions installed through
    * clutter_threads_add_repaint_func()
    */
@@ -172,48 +184,27 @@ typedef struct
 gboolean _clutter_threads_dispatch      (gpointer data);
 void     _clutter_threads_dispatch_free (gpointer data);
 
-#define CLUTTER_CONTEXT()	(_clutter_context_get_default ())
 ClutterMainContext *    _clutter_context_get_default            (void);
 gboolean                _clutter_context_is_initialized         (void);
-PangoContext *          _clutter_context_create_pango_context   (ClutterMainContext *self);
-PangoContext *          _clutter_context_get_pango_context      (ClutterMainContext *self);
-
-#define CLUTTER_PARAM_READABLE  (G_PARAM_READABLE | G_PARAM_STATIC_STRINGS)
-#define CLUTTER_PARAM_WRITABLE  (G_PARAM_WRITABLE | G_PARAM_STATIC_STRINGS)
-#define CLUTTER_PARAM_READWRITE (G_PARAM_READABLE | G_PARAM_WRITABLE | G_PARAM_STATIC_STRINGS)
-
-/* automagic interning of a static string */
-#define I_(str)  (g_intern_static_string ((str)))
-
-/* mark all properties under the "Property" context */
-#ifdef ENABLE_NLS
-#define P_(String) (_clutter_gettext ((String)))
-#else
-#define P_(String) (String)
-#endif
+PangoContext *          _clutter_context_create_pango_context   (void);
+PangoContext *          _clutter_context_get_pango_context      (void);
+ClutterPickMode         _clutter_context_get_pick_mode          (void);
+void                    _clutter_context_push_shader_stack      (ClutterActor *actor);
+ClutterActor *          _clutter_context_pop_shader_stack       (ClutterActor *actor);
+ClutterActor *          _clutter_context_peek_shader_stack      (void);
+guint32                 _clutter_context_acquire_id             (gpointer      key);
+void                    _clutter_context_release_id             (guint32       id_);
 
 G_CONST_RETURN gchar *_clutter_gettext (const gchar *str);
 
 gboolean      _clutter_feature_init (GError **error);
 
-/* Reinjecting queued events for processing */
-void _clutter_process_event (ClutterEvent *event);
-
-/* clears the event queue inside the main context */
-void _clutter_clear_events_queue           (void);
-void _clutter_clear_events_queue_for_stage (ClutterStage *stage);
-
 /* Picking code */
 ClutterActor *_clutter_do_pick (ClutterStage    *stage,
 				gint             x,
 				gint             y,
 				ClutterPickMode  mode);
 
-/* the actual redraw */
-void          _clutter_do_redraw (ClutterStage *stage);
-
-guint         _clutter_pixel_to_id (guchar pixel[4]);
-
 void          _clutter_id_to_color (guint id,
                                     ClutterColor *col);
 
@@ -228,34 +219,18 @@ gboolean _clutter_boolean_handled_accumulator (GSignalInvocationHint *ihint,
 
 void _clutter_run_repaint_functions (void);
 
-gboolean _clutter_effect_pre_paint        (ClutterEffect      *effect);
-void     _clutter_effect_post_paint       (ClutterEffect      *effect);
-gboolean _clutter_effect_get_paint_volume (ClutterEffect      *effect,
-                                           ClutterPaintVolume *volume);
-
 void _clutter_constraint_update_allocation (ClutterConstraint *constraint,
                                             ClutterActor      *actor,
                                             ClutterActorBox   *allocation);
 
 GType _clutter_layout_manager_get_child_meta_type (ClutterLayoutManager *manager);
 
-void     _clutter_event_set_platform_data (ClutterEvent       *event,
-                                           gpointer            data);
-gpointer _clutter_event_get_platform_data (const ClutterEvent *event);
-
-void     _clutter_event_push              (const ClutterEvent *event,
-                                           gboolean            do_copy);
-void     _clutter_event_set_device        (ClutterEvent       *event,
-                                           ClutterInputDevice *device);
-void     _clutter_event_set_source_device (ClutterEvent       *event,
-                                           ClutterInputDevice *device);
-
-void                _clutter_util_fully_transform_vertices (const CoglMatrix *modelview,
-                                                            const CoglMatrix *projection,
-                                                            const int *viewport,
-                                                            const ClutterVertex *vertices_in,
-                                                            ClutterVertex *vertices_out,
-                                                            int n_vertices);
+void  _clutter_util_fully_transform_vertices (const CoglMatrix    *modelview,
+                                              const CoglMatrix    *projection,
+                                              const int           *viewport,
+                                              const ClutterVertex *vertices_in,
+                                              ClutterVertex       *vertices_out,
+                                              int                  n_vertices);
 
 G_END_DECLS
 
