@@ -29,14 +29,14 @@
 #define __CLUTTER_TYPES_H__
 
 #include <glib-object.h>
-#include <clutter/clutter-units.h>
 
 G_BEGIN_DECLS
 
-#define CLUTTER_TYPE_ACTOR_BOX  (clutter_actor_box_get_type ())
-#define CLUTTER_TYPE_GEOMETRY   (clutter_geometry_get_type ())
-#define CLUTTER_TYPE_KNOT       (clutter_knot_get_type ())
-#define CLUTTER_TYPE_VERTEX     (clutter_vertex_get_type ())
+#define CLUTTER_TYPE_ACTOR_BOX          (clutter_actor_box_get_type ())
+#define CLUTTER_TYPE_GEOMETRY           (clutter_geometry_get_type ())
+#define CLUTTER_TYPE_KNOT               (clutter_knot_get_type ())
+#define CLUTTER_TYPE_PAINT_VOLUME       (clutter_paint_volume_get_type ())
+#define CLUTTER_TYPE_VERTEX             (clutter_vertex_get_type ())
 
 /* Forward delarations to avoid header catch 22's */
 typedef struct _ClutterActor            ClutterActor;
@@ -51,6 +51,11 @@ typedef struct _ClutterAnimator         ClutterAnimator;
 
 typedef struct _ClutterAction           ClutterAction;
 typedef struct _ClutterConstraint       ClutterConstraint;
+typedef struct _ClutterEffect           ClutterEffect;
+
+typedef struct _ClutterShader           ClutterShader;
+
+typedef struct _ClutterColor            ClutterColor;
 
 typedef union _ClutterEvent             ClutterEvent;
 
@@ -91,6 +96,23 @@ typedef struct _ClutterGeometry         ClutterGeometry;
 typedef struct _ClutterKnot             ClutterKnot;
 typedef struct _ClutterVertex           ClutterVertex;
 
+/**
+ * ClutterPaintVolume:
+ *
+ * <structname>ClutterPaintVolume</structname> is an opaque structure
+ * whose members cannot be directly accessed.
+ *
+ * A <structname>ClutterPaintVolume</structname> represents an
+ * a bounding volume whos internal representation isn't defined but
+ * can be set and queried in terms of an axis aligned bounding box.
+ *
+ * Other internal representation and methods for describing the
+ * bounding volume may be added in the future.
+ *
+ * Since: 1.4
+ */
+typedef struct _ClutterPaintVolume      ClutterPaintVolume;
+
 /**
  * ClutterVertex:
  * @x: X coordinate of the vertex
@@ -167,6 +189,16 @@ void             clutter_actor_box_interpolate   (const ClutterActorBox *initial
                                                   gdouble                progress,
                                                   ClutterActorBox       *result);
 void             clutter_actor_box_clamp_to_pixel (ClutterActorBox       *box);
+void             clutter_actor_box_union          (const ClutterActorBox *a,
+                                                   const ClutterActorBox *b,
+                                                   ClutterActorBox       *result);
+
+void             clutter_actor_box_set_origin     (ClutterActorBox       *box,
+                                                   gfloat                 x,
+                                                   gfloat                 y);
+void             clutter_actor_box_set_size       (ClutterActorBox       *box,
+                                                   gfloat                 width,
+                                                   gfloat                 height);
 
 /**
  * ClutterGeometry:
@@ -433,6 +465,81 @@ typedef enum {
   CLUTTER_FRAGMENT_SHADER
 } ClutterShaderType;
 
+GType clutter_paint_volume_get_type (void) G_GNUC_CONST;
+
+ClutterPaintVolume *clutter_paint_volume_copy                (const ClutterPaintVolume *pv);
+void                clutter_paint_volume_free                (ClutterPaintVolume       *pv);
+
+void                clutter_paint_volume_set_origin          (ClutterPaintVolume       *pv,
+                                                              const ClutterVertex      *origin);
+void                clutter_paint_volume_get_origin          (const ClutterPaintVolume *pv,
+                                                              ClutterVertex            *vertex);
+void                clutter_paint_volume_set_width           (ClutterPaintVolume       *pv,
+                                                              gfloat                    width);
+gfloat              clutter_paint_volume_get_width           (const ClutterPaintVolume *pv);
+void                clutter_paint_volume_set_height          (ClutterPaintVolume       *pv,
+                                                              gfloat                    height);
+gfloat              clutter_paint_volume_get_height          (const ClutterPaintVolume *pv);
+void                clutter_paint_volume_set_depth           (ClutterPaintVolume       *pv,
+                                                              gfloat                    depth);
+gfloat              clutter_paint_volume_get_depth           (const ClutterPaintVolume *pv);
+void                clutter_paint_volume_union               (ClutterPaintVolume       *pv,
+                                                              const ClutterPaintVolume *another_pv);
+
+gboolean            clutter_paint_volume_set_from_allocation (ClutterPaintVolume       *pv,
+                                                              ClutterActor             *actor);
+
+/**
+ * ClutterModifierType:
+ * @CLUTTER_SHIFT_MASK: Mask applied by the Shift key
+ * @CLUTTER_LOCK_MASK: Mask applied by the Caps Lock key
+ * @CLUTTER_CONTROL_MASK: Mask applied by the Control key
+ * @CLUTTER_MOD1_MASK: Mask applied by the first Mod key
+ * @CLUTTER_MOD2_MASK: Mask applied by the second Mod key
+ * @CLUTTER_MOD3_MASK: Mask applied by the third Mod key
+ * @CLUTTER_MOD4_MASK: Mask applied by the fourth Mod key
+ * @CLUTTER_MOD5_MASK: Mask applied by the fifth Mod key
+ * @CLUTTER_BUTTON1_MASK: Mask applied by the first pointer button
+ * @CLUTTER_BUTTON2_MASK: Mask applied by the second pointer button
+ * @CLUTTER_BUTTON3_MASK: Mask applied by the third pointer button
+ * @CLUTTER_BUTTON4_MASK: Mask applied by the fourth pointer button
+ * @CLUTTER_BUTTON5_MASK: Mask applied by the fifth pointer button
+ * @CLUTTER_SUPER_MASK: Mask applied by the Super key
+ * @CLUTTER_HYPER_MASK: Mask applied by the Hyper key
+ * @CLUTTER_META_MASK: Mask applied by the Meta key
+ * @CLUTTER_RELEASE_MASK: Mask applied during release
+ * @CLUTTER_MODIFIER_MASK: A mask covering all modifier types
+ *
+ * Masks applied to a #ClutterEvent by modifiers.
+ *
+ * Since: 0.4
+ */
+typedef enum {
+  CLUTTER_SHIFT_MASK    = 1 << 0,
+  CLUTTER_LOCK_MASK     = 1 << 1,
+  CLUTTER_CONTROL_MASK  = 1 << 2,
+  CLUTTER_MOD1_MASK     = 1 << 3,
+  CLUTTER_MOD2_MASK     = 1 << 4,
+  CLUTTER_MOD3_MASK     = 1 << 5,
+  CLUTTER_MOD4_MASK     = 1 << 6,
+  CLUTTER_MOD5_MASK     = 1 << 7,
+  CLUTTER_BUTTON1_MASK  = 1 << 8,
+  CLUTTER_BUTTON2_MASK  = 1 << 9,
+  CLUTTER_BUTTON3_MASK  = 1 << 10,
+  CLUTTER_BUTTON4_MASK  = 1 << 11,
+  CLUTTER_BUTTON5_MASK  = 1 << 12,
+
+  /* bits 15 to 25 are currently unused; bit 29 is used internally */
+
+  CLUTTER_SUPER_MASK    = 1 << 26,
+  CLUTTER_HYPER_MASK    = 1 << 27,
+  CLUTTER_META_MASK     = 1 << 28,
+
+  CLUTTER_RELEASE_MASK  = 1 << 30,
+
+  CLUTTER_MODIFIER_MASK = 0x5c001fff
+} ClutterModifierType;
+
 G_END_DECLS
 
 #endif /* __CLUTTER_TYPES_H__ */
