@@ -480,7 +480,7 @@ enable_gl_state (CoglDrawFlags flags,
                  CoglAttribute **attributes,
                  ValidateLayerState *state)
 {
-  CoglFramebuffer *framebuffer = _cogl_get_framebuffer ();
+  CoglFramebuffer *framebuffer = _cogl_get_draw_buffer ();
   int i;
 #ifdef MAY_HAVE_PROGRAMABLE_GL
   GLuint generic_index = 0;
@@ -600,7 +600,7 @@ enable_gl_state (CoglDrawFlags flags,
       CoglAttribute *attribute = attributes[i];
       CoglVertexArray *vertex_array;
       CoglBuffer *buffer;
-      void *base;
+      guint8 *base;
 #ifdef HAVE_COGL_GLES2
       int attrib_location;
 #endif
@@ -882,7 +882,7 @@ get_wire_lines (CoglAttribute *attribute,
   CoglIndicesType indices_type;
   int i;
   int n_lines;
-  CoglVertexP3 *out;
+  CoglVertexP3 *out = NULL;
 
   vertices = cogl_buffer_map (COGL_BUFFER (vertex_array),
                               COGL_BUFFER_ACCESS_READ, 0);
@@ -894,7 +894,11 @@ get_wire_lines (CoglAttribute *attribute,
       indices_type = cogl_indices_get_type (_indices);
     }
   else
-    indices = NULL;
+    {
+      index_array = NULL;
+      indices = NULL;
+      indices_type = COGL_INDICES_TYPE_UNSIGNED_BYTE;
+    }
 
   *n_vertices_out = 0;
 
@@ -977,10 +981,12 @@ get_wire_lines (CoglAttribute *attribute,
     }
 #endif
 
-  if (vertices)
+  if (vertices != NULL)
     cogl_buffer_unmap (COGL_BUFFER (vertex_array));
-  if (indices)
+
+  if (indices != NULL)
     cogl_buffer_unmap (COGL_BUFFER (index_array));
+
   return out;
 }
 
@@ -1036,7 +1042,7 @@ draw_wireframe (CoglVerticesMode mode,
   cogl_push_source (wire_pipeline);
 
   /* temporarily disable the wireframe to avoid recursion! */
-  cogl_debug_flags &= ~COGL_DEBUG_WIREFRAME;
+  COGL_DEBUG_CLEAR_FLAG (COGL_DEBUG_WIREFRAME);
   _cogl_draw_attributes_array (COGL_VERTICES_MODE_LINES,
                                0,
                                n_line_vertices,
@@ -1046,7 +1052,7 @@ draw_wireframe (CoglVerticesMode mode,
                                COGL_DRAW_SKIP_FRAMEBUFFER_FLUSH |
                                COGL_DRAW_SKIP_LEGACY_STATE);
 
-  cogl_debug_flags |= COGL_DEBUG_WIREFRAME;
+  COGL_DEBUG_SET_FLAG (COGL_DEBUG_WIREFRAME);
 
   cogl_pop_source ();
 
@@ -1060,7 +1066,7 @@ flush_state (CoglDrawFlags flags,
 {
   if (!(flags & COGL_DRAW_SKIP_JOURNAL_FLUSH))
     {
-      CoglFramebuffer *framebuffer = _cogl_get_framebuffer ();
+      CoglFramebuffer *framebuffer = _cogl_get_draw_buffer ();
       _cogl_journal_flush (framebuffer->journal, framebuffer);
     }
 
@@ -1080,7 +1086,9 @@ flush_state (CoglDrawFlags flags,
    * stack can cause some drawing which would change the array
    * pointers. */
   if (!(flags & COGL_DRAW_SKIP_FRAMEBUFFER_FLUSH))
-    _cogl_framebuffer_flush_state (_cogl_get_framebuffer (), 0);
+    _cogl_framebuffer_flush_state (_cogl_get_draw_buffer (),
+                                   _cogl_get_read_buffer (),
+                                   0);
 }
 
 /* This can be called directly by the CoglJournal to draw attributes
@@ -1107,7 +1115,7 @@ _cogl_draw_attributes_array (CoglVerticesMode mode,
   disable_gl_state (attributes, source);
 
 #ifdef COGL_ENABLE_DEBUG
-  if (G_UNLIKELY (cogl_debug_flags & COGL_DEBUG_WIREFRAME))
+  if (G_UNLIKELY (COGL_DEBUG_ENABLED (COGL_DEBUG_WIREFRAME)))
     draw_wireframe (mode, first_vertex, n_vertices, attributes, NULL);
 #endif
 }
@@ -1178,7 +1186,7 @@ _cogl_draw_indexed_attributes_array (CoglVerticesMode mode,
   ValidateLayerState state;
   CoglPipeline *source;
   CoglBuffer *buffer;
-  void *base;
+  guint8 *base;
   size_t array_offset;
   size_t index_size;
   GLenum indices_gl_type = 0;
@@ -1219,7 +1227,7 @@ _cogl_draw_indexed_attributes_array (CoglVerticesMode mode,
   disable_gl_state (attributes, source);
 
 #ifdef COGL_ENABLE_DEBUG
-  if (G_UNLIKELY (cogl_debug_flags & COGL_DEBUG_WIREFRAME))
+  if (G_UNLIKELY (COGL_DEBUG_ENABLED (COGL_DEBUG_WIREFRAME)))
     draw_wireframe (mode, first_vertex, n_vertices, attributes, indices);
 #endif
 }
