@@ -16,9 +16,9 @@
  * Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public
- * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * License along with this library. If not, see <http://www.gnu.org/licenses/>.
+ *
+ *
  */
 
 #ifdef HAVE_CONFIG_H
@@ -33,7 +33,8 @@
 #include "cogl-feature-private.h"
 
 gboolean
-_cogl_feature_check (const CoglFeatureData *data,
+_cogl_feature_check (const char *driver_prefix,
+                     const CoglFeatureData *data,
                      unsigned int gl_major,
                      unsigned int gl_minor,
                      const char *extensions_string)
@@ -79,8 +80,8 @@ _cogl_feature_check (const CoglFeatureData *data,
                *extension;
                extension += strlen (extension) + 1)
             {
-              g_string_set_size (full_extension_name, 0);
-              g_string_append (full_extension_name, "GL_");
+              g_string_assign (full_extension_name, driver_prefix);
+              g_string_append_c (full_extension_name, '_');
               g_string_append_len (full_extension_name,
                                    namespace, namespace_len);
               g_string_append_c (full_extension_name, '_');
