@@ -42,79 +42,213 @@ typedef struct _CoglPipelineLayer     CoglPipelineLayer;
 
 #if defined (HAVE_COGL_GL)
 
-/* NB: pipeline->backend is currently a 3bit unsigned int bitfield */
-#define COGL_PIPELINE_BACKEND_ARBFP      0
-#define COGL_PIPELINE_BACKEND_ARBFP_MASK (1L<<0)
-#define COGL_PIPELINE_BACKEND_FIXED      1
-#define COGL_PIPELINE_BACKEND_FIXED_MASK (1L<<1)
-#define COGL_PIPELINE_BACKEND_GLSL       2
-#define COGL_PIPELINE_BACKEND_GLSL_MASK  (1L<<2)
+/* NB: pipeline->fragend is currently a 3bit unsigned int bitfield */
+#define COGL_PIPELINE_FRAGEND_ARBFP      0
+#define COGL_PIPELINE_FRAGEND_ARBFP_MASK (1L<<0)
+#define COGL_PIPELINE_FRAGEND_FIXED      1
+#define COGL_PIPELINE_FRAGEND_FIXED_MASK (1L<<1)
+#define COGL_PIPELINE_FRAGEND_GLSL       2
+#define COGL_PIPELINE_FRAGEND_GLSL_MASK  (1L<<2)
 
-#define COGL_PIPELINE_N_BACKENDS         3
+#define COGL_PIPELINE_N_FRAGENDS         3
 
 #elif defined (HAVE_COGL_GLES2)
 
-#define COGL_PIPELINE_BACKEND_GLSL       0
-#define COGL_PIPELINE_BACKEND_GLSL_MASK  (1L<<0)
+#define COGL_PIPELINE_FRAGEND_GLSL       0
+#define COGL_PIPELINE_FRAGEND_GLSL_MASK  (1L<<0)
 
-#define COGL_PIPELINE_N_BACKENDS         1
+#define COGL_PIPELINE_N_FRAGENDS         1
 
 #else /* HAVE_COGL_GLES */
 
-#define COGL_PIPELINE_BACKEND_FIXED      0
-#define COGL_PIPELINE_BACKEND_FIXED_MASK (1L<<0)
+#define COGL_PIPELINE_FRAGEND_FIXED      0
+#define COGL_PIPELINE_FRAGEND_FIXED_MASK (1L<<0)
 
-#define COGL_PIPELINE_N_BACKENDS         1
+#define COGL_PIPELINE_N_FRAGENDS         1
 
 #endif
 
-#define COGL_PIPELINE_BACKEND_DEFAULT    0
-#define COGL_PIPELINE_BACKEND_UNDEFINED  3
+#define COGL_PIPELINE_FRAGEND_DEFAULT    0
+#define COGL_PIPELINE_FRAGEND_UNDEFINED  3
 
+#if defined (HAVE_COGL_GL)
+
+#define COGL_PIPELINE_VERTEND_FIXED      0
+#define COGL_PIPELINE_VERTEND_GLSL       1
+
+#define COGL_PIPELINE_N_VERTENDS         2
+
+#elif defined (HAVE_COGL_GLES2)
+
+#define COGL_PIPELINE_VERTEND_GLSL       0
+
+#define COGL_PIPELINE_N_VERTENDS         1
+
+#else /* HAVE_COGL_GLES */
+
+#define COGL_PIPELINE_VERTEND_FIXED      0
+
+#define COGL_PIPELINE_N_VERTENDS         1
+
+#endif
+
+#define COGL_PIPELINE_VERTEND_DEFAULT    0
+#define COGL_PIPELINE_VERTEND_UNDEFINED  3
+
+/* If we have either of the GLSL backends then we also need a GLSL
+   progend to combine the shaders generated into a single
+   program. Currently there is only one progend but if we ever add
+   other languages they would likely need their own progend too. The
+   progends are different from the other backends because there can be
+   more than one in use for each pipeline. All of the progends are
+   invoked whenever a pipeline is flushed. */
+#ifdef COGL_PIPELINE_FRAGEND_GLSL
+#define COGL_PIPELINE_PROGEND_GLSL       0
+#define COGL_PIPELINE_N_PROGENDS         1
+#else
+#define COGL_PIPELINE_N_PROGENDS         0
+#endif
+
+/* XXX: should I rename these as
+ * COGL_PIPELINE_LAYER_STATE_INDEX_XYZ... ?
+ */
 typedef enum
 {
-  COGL_PIPELINE_LAYER_STATE_UNIT             = 1L<<0,
-  COGL_PIPELINE_LAYER_STATE_TEXTURE          = 1L<<1,
-  COGL_PIPELINE_LAYER_STATE_FILTERS          = 1L<<2,
-  COGL_PIPELINE_LAYER_STATE_WRAP_MODES       = 1L<<3,
+  /* sparse state */
+  COGL_PIPELINE_LAYER_STATE_UNIT_INDEX,
+  COGL_PIPELINE_LAYER_STATE_TEXTURE_TARGET_INDEX,
+  COGL_PIPELINE_LAYER_STATE_TEXTURE_DATA_INDEX,
+  COGL_PIPELINE_LAYER_STATE_FILTERS_INDEX,
+  COGL_PIPELINE_LAYER_STATE_WRAP_MODES_INDEX,
+  COGL_PIPELINE_LAYER_STATE_COMBINE_INDEX,
+  COGL_PIPELINE_LAYER_STATE_COMBINE_CONSTANT_INDEX,
+  COGL_PIPELINE_LAYER_STATE_USER_MATRIX_INDEX,
+  COGL_PIPELINE_LAYER_STATE_POINT_SPRITE_COORDS_INDEX,
 
-  COGL_PIPELINE_LAYER_STATE_COMBINE          = 1L<<4,
-  COGL_PIPELINE_LAYER_STATE_COMBINE_CONSTANT = 1L<<5,
-  COGL_PIPELINE_LAYER_STATE_USER_MATRIX      = 1L<<6,
+  /* note: layers don't currently have any non-sparse state */
 
-  COGL_PIPELINE_LAYER_STATE_POINT_SPRITE_COORDS = 1L<<7,
+  COGL_PIPELINE_LAYER_STATE_SPARSE_COUNT,
+  COGL_PIPELINE_LAYER_STATE_COUNT = COGL_PIPELINE_LAYER_STATE_SPARSE_COUNT
+} CoglPipelineLayerStateIndex;
+
+/* XXX: If you add or remove state groups here you may need to update
+ * some of the state masks following this enum too!
+ *
+ * FIXME: perhaps it would be better to rename this enum to
+ * CoglPipelineLayerStateGroup to better convey the fact that a single
+ * enum here can map to multiple properties.
+ */
+typedef enum
+{
+  COGL_PIPELINE_LAYER_STATE_UNIT =
+    1L<<COGL_PIPELINE_LAYER_STATE_UNIT_INDEX,
+  COGL_PIPELINE_LAYER_STATE_TEXTURE_TARGET =
+    1L<<COGL_PIPELINE_LAYER_STATE_TEXTURE_TARGET_INDEX,
+  COGL_PIPELINE_LAYER_STATE_TEXTURE_DATA =
+    1L<<COGL_PIPELINE_LAYER_STATE_TEXTURE_DATA_INDEX,
+  COGL_PIPELINE_LAYER_STATE_FILTERS =
+    1L<<COGL_PIPELINE_LAYER_STATE_FILTERS_INDEX,
+  COGL_PIPELINE_LAYER_STATE_WRAP_MODES =
+    1L<<COGL_PIPELINE_LAYER_STATE_WRAP_MODES_INDEX,
+
+  COGL_PIPELINE_LAYER_STATE_COMBINE =
+    1L<<COGL_PIPELINE_LAYER_STATE_COMBINE_INDEX,
+  COGL_PIPELINE_LAYER_STATE_COMBINE_CONSTANT =
+    1L<<COGL_PIPELINE_LAYER_STATE_COMBINE_CONSTANT_INDEX,
+  COGL_PIPELINE_LAYER_STATE_USER_MATRIX =
+    1L<<COGL_PIPELINE_LAYER_STATE_USER_MATRIX_INDEX,
+
+  COGL_PIPELINE_LAYER_STATE_POINT_SPRITE_COORDS =
+    1L<<COGL_PIPELINE_LAYER_STATE_POINT_SPRITE_COORDS_INDEX,
 
   /* COGL_PIPELINE_LAYER_STATE_TEXTURE_INTERN   = 1L<<8, */
 
-  COGL_PIPELINE_LAYER_STATE_ALL_SPARSE =
-    COGL_PIPELINE_LAYER_STATE_UNIT |
-    COGL_PIPELINE_LAYER_STATE_TEXTURE |
-    COGL_PIPELINE_LAYER_STATE_FILTERS |
-    COGL_PIPELINE_LAYER_STATE_WRAP_MODES |
-    COGL_PIPELINE_LAYER_STATE_COMBINE |
-    COGL_PIPELINE_LAYER_STATE_COMBINE_CONSTANT |
-    COGL_PIPELINE_LAYER_STATE_USER_MATRIX |
-    COGL_PIPELINE_LAYER_STATE_POINT_SPRITE_COORDS,
-
-  COGL_PIPELINE_LAYER_STATE_NEEDS_BIG_STATE =
-    COGL_PIPELINE_LAYER_STATE_COMBINE |
-    COGL_PIPELINE_LAYER_STATE_COMBINE_CONSTANT |
-    COGL_PIPELINE_LAYER_STATE_USER_MATRIX |
-    COGL_PIPELINE_LAYER_STATE_POINT_SPRITE_COORDS,
-
 } CoglPipelineLayerState;
 
+/*
+ * Various special masks that tag state-groups in different ways...
+ */
+
+#define COGL_PIPELINE_LAYER_STATE_ALL \
+  ((1L<<COGL_PIPELINE_LAYER_STATE_COUNT) - 1)
+
+#define COGL_PIPELINE_LAYER_STATE_ALL_SPARSE \
+  COGL_PIPELINE_LAYER_STATE_ALL
+
+#define COGL_PIPELINE_LAYER_STATE_NEEDS_BIG_STATE \
+  (COGL_PIPELINE_LAYER_STATE_COMBINE | \
+   COGL_PIPELINE_LAYER_STATE_COMBINE_CONSTANT | \
+   COGL_PIPELINE_LAYER_STATE_USER_MATRIX | \
+   COGL_PIPELINE_LAYER_STATE_POINT_SPRITE_COORDS)
+
+#define COGL_PIPELINE_LAYER_STATE_MULTI_PROPERTY \
+  (COGL_PIPELINE_LAYER_STATE_FILTERS | \
+   COGL_PIPELINE_LAYER_STATE_WRAP_MODES | \
+   COGL_PIPELINE_LAYER_STATE_COMBINE)
+
+/* FIXME: Only texture target changes should really affect the
+ * codegen, but this is difficult to detect */
+#ifdef HAVE_COGL_GLES2
+/* On GLES2 we need to use a different varying for the texture lookups
+ * when point sprite coords are enabled */
+#define COGL_PIPELINE_LAYER_STATE_AFFECTS_FRAGMENT_CODEGEN \
+  (COGL_PIPELINE_LAYER_STATE_COMBINE | \
+   COGL_PIPELINE_LAYER_STATE_TEXTURE_TARGET | \
+   COGL_PIPELINE_LAYER_STATE_POINT_SPRITE_COORDS | \
+   COGL_PIPELINE_LAYER_STATE_UNIT)
+#else
+#define COGL_PIPELINE_LAYER_STATE_AFFECTS_FRAGMENT_CODEGEN \
+  (COGL_PIPELINE_LAYER_STATE_COMBINE | \
+   COGL_PIPELINE_LAYER_STATE_TEXTURE_TARGET | \
+   COGL_PIPELINE_LAYER_STATE_UNIT)
+#endif
+
+#define COGL_PIPELINE_LAYER_STATE_AFFECTS_VERTEX_CODEGEN 0
+
+
+typedef enum
+{
+  /* These are the same values as GL */
+  COGL_PIPELINE_COMBINE_FUNC_ADD         = 0x0104,
+  COGL_PIPELINE_COMBINE_FUNC_ADD_SIGNED  = 0x8574,
+  COGL_PIPELINE_COMBINE_FUNC_SUBTRACT    = 0x84E7,
+  COGL_PIPELINE_COMBINE_FUNC_INTERPOLATE = 0x8575,
+  COGL_PIPELINE_COMBINE_FUNC_REPLACE     = 0x1E01,
+  COGL_PIPELINE_COMBINE_FUNC_MODULATE    = 0x2100,
+  COGL_PIPELINE_COMBINE_FUNC_DOT3_RGB    = 0x86AE,
+  COGL_PIPELINE_COMBINE_FUNC_DOT3_RGBA   = 0x86AF
+} CoglPipelineCombineFunc;
+
+typedef enum
+{
+  /* These are the same values as GL */
+  COGL_PIPELINE_COMBINE_SOURCE_TEXTURE       = 0x1702,
+  COGL_PIPELINE_COMBINE_SOURCE_CONSTANT      = 0x8576,
+  COGL_PIPELINE_COMBINE_SOURCE_PRIMARY_COLOR = 0x8577,
+  COGL_PIPELINE_COMBINE_SOURCE_PREVIOUS      = 0x8578,
+  COGL_PIPELINE_COMBINE_SOURCE_TEXTURE0      = 0x84C0
+} CoglPipelineCombineSource;
+
+typedef enum
+{
+  /* These are the same values as GL */
+  COGL_PIPELINE_COMBINE_OP_SRC_COLOR           = 0x0300,
+  COGL_PIPELINE_COMBINE_OP_ONE_MINUS_SRC_COLOR = 0x0301,
+  COGL_PIPELINE_COMBINE_OP_SRC_ALPHA           = 0x0302,
+  COGL_PIPELINE_COMBINE_OP_ONE_MINUS_SRC_ALPHA = 0x0303
+} CoglPipelineCombineOp;
+
 typedef struct
 {
   /* The texture combine state determines how the color of individual
    * texture fragments are calculated. */
-  GLint texture_combine_rgb_func;
-  GLint texture_combine_rgb_src[3];
-  GLint texture_combine_rgb_op[3];
+  CoglPipelineCombineFunc texture_combine_rgb_func;
+  CoglPipelineCombineSource texture_combine_rgb_src[3];
+  CoglPipelineCombineOp texture_combine_rgb_op[3];
 
-  GLint texture_combine_alpha_func;
-  GLint texture_combine_alpha_src[3];
-  GLint texture_combine_alpha_op[3];
+  CoglPipelineCombineFunc texture_combine_alpha_func;
+  CoglPipelineCombineSource texture_combine_alpha_src[3];
+  CoglPipelineCombineOp texture_combine_alpha_op[3];
 
   float texture_combine_constant[4];
 
@@ -221,14 +355,6 @@ struct _CoglPipelineLayer
   /* The lowest index is blended first then others on top */
   int	             index;
 
-  /* Different pipeline backends (GLSL/ARBfp/Fixed Function) may
-   * want to associate private data with a layer...
-   *
-   * NB: we have per backend pointers because a layer may be
-   * associated with multiple pipelines with different backends.
-   */
-  void              *backend_priv[COGL_PIPELINE_N_BACKENDS];
-
   /* A mask of which state groups are different in this layer
    * in comparison to its parent. */
   unsigned long             differences;
@@ -247,6 +373,7 @@ struct _CoglPipelineLayer
   /* The texture for this layer, or COGL_INVALID_HANDLE for an empty
    * layer */
   CoglHandle                 texture;
+  GLenum                     target;
 
   CoglPipelineFilter         mag_filter;
   CoglPipelineFilter         min_filter;
@@ -267,58 +394,125 @@ struct _CoglPipelineLayer
 
 };
 
+/* XXX: should I rename these as
+ * COGL_PIPELINE_STATE_INDEX_XYZ... ?
+ */
+typedef enum
+{
+  /* sparse state */
+  COGL_PIPELINE_STATE_COLOR_INDEX,
+  COGL_PIPELINE_STATE_BLEND_ENABLE_INDEX,
+  COGL_PIPELINE_STATE_LAYERS_INDEX,
+  COGL_PIPELINE_STATE_LIGHTING_INDEX,
+  COGL_PIPELINE_STATE_ALPHA_FUNC_INDEX,
+  COGL_PIPELINE_STATE_ALPHA_FUNC_REFERENCE_INDEX,
+  COGL_PIPELINE_STATE_BLEND_INDEX,
+  COGL_PIPELINE_STATE_USER_SHADER_INDEX,
+  COGL_PIPELINE_STATE_DEPTH_INDEX,
+  COGL_PIPELINE_STATE_FOG_INDEX,
+  COGL_PIPELINE_STATE_POINT_SIZE_INDEX,
+
+  /* non-sparse */
+  COGL_PIPELINE_STATE_REAL_BLEND_ENABLE_INDEX,
+
+  COGL_PIPELINE_STATE_COUNT,
+  COGL_PIPELINE_STATE_SPARSE_COUNT = COGL_PIPELINE_STATE_COUNT - 1,
+} CoglPipelineStateIndex;
+
 /* Used in pipeline->differences masks and for notifying pipeline
- * state changes... */
+ * state changes.
+ *
+ * XXX: If you add or remove state groups here you may need to update
+ * some of the state masks following this enum too!
+ *
+ * FIXME: perhaps it would be better to rename this enum to
+ * CoglPipelineStateGroup to better convey the fact that a single enum
+ * here can map to multiple properties.
+ */
 typedef enum _CoglPipelineState
 {
-  COGL_PIPELINE_STATE_COLOR                = 1L<<0,
-  COGL_PIPELINE_STATE_BLEND_ENABLE         = 1L<<1,
-  COGL_PIPELINE_STATE_LAYERS               = 1L<<2,
+  COGL_PIPELINE_STATE_COLOR =
+    1L<<COGL_PIPELINE_STATE_COLOR_INDEX,
+  COGL_PIPELINE_STATE_BLEND_ENABLE =
+    1L<<COGL_PIPELINE_STATE_BLEND_ENABLE_INDEX,
+  COGL_PIPELINE_STATE_LAYERS =
+    1L<<COGL_PIPELINE_STATE_LAYERS_INDEX,
 
-  COGL_PIPELINE_STATE_LIGHTING             = 1L<<3,
-  COGL_PIPELINE_STATE_ALPHA_FUNC           = 1L<<4,
-  COGL_PIPELINE_STATE_ALPHA_FUNC_REFERENCE = 1L<<5,
-  COGL_PIPELINE_STATE_BLEND                = 1L<<6,
-  COGL_PIPELINE_STATE_USER_SHADER          = 1L<<7,
-  COGL_PIPELINE_STATE_DEPTH                = 1L<<8,
-  COGL_PIPELINE_STATE_FOG                  = 1L<<9,
-  COGL_PIPELINE_STATE_POINT_SIZE           = 1L<<10,
+  COGL_PIPELINE_STATE_LIGHTING =
+    1L<<COGL_PIPELINE_STATE_LIGHTING_INDEX,
+  COGL_PIPELINE_STATE_ALPHA_FUNC =
+    1L<<COGL_PIPELINE_STATE_ALPHA_FUNC_INDEX,
+  COGL_PIPELINE_STATE_ALPHA_FUNC_REFERENCE =
+    1L<<COGL_PIPELINE_STATE_ALPHA_FUNC_REFERENCE_INDEX,
+  COGL_PIPELINE_STATE_BLEND =
+    1L<<COGL_PIPELINE_STATE_BLEND_INDEX,
+  COGL_PIPELINE_STATE_USER_SHADER =
+    1L<<COGL_PIPELINE_STATE_USER_SHADER_INDEX,
+  COGL_PIPELINE_STATE_DEPTH =
+    1L<<COGL_PIPELINE_STATE_DEPTH_INDEX,
+  COGL_PIPELINE_STATE_FOG =
+    1L<<COGL_PIPELINE_STATE_FOG_INDEX,
+  COGL_PIPELINE_STATE_POINT_SIZE =
+    1L<<COGL_PIPELINE_STATE_POINT_SIZE_INDEX,
 
-  COGL_PIPELINE_STATE_REAL_BLEND_ENABLE    = 1L<<11,
-
-  COGL_PIPELINE_STATE_ALL_SPARSE =
-    COGL_PIPELINE_STATE_COLOR |
-    COGL_PIPELINE_STATE_BLEND_ENABLE |
-    COGL_PIPELINE_STATE_LAYERS |
-    COGL_PIPELINE_STATE_LIGHTING |
-    COGL_PIPELINE_STATE_ALPHA_FUNC |
-    COGL_PIPELINE_STATE_ALPHA_FUNC_REFERENCE |
-    COGL_PIPELINE_STATE_BLEND |
-    COGL_PIPELINE_STATE_USER_SHADER |
-    COGL_PIPELINE_STATE_DEPTH |
-    COGL_PIPELINE_STATE_FOG |
-    COGL_PIPELINE_STATE_POINT_SIZE,
-
-  COGL_PIPELINE_STATE_AFFECTS_BLENDING =
-    COGL_PIPELINE_STATE_COLOR |
-    COGL_PIPELINE_STATE_BLEND_ENABLE |
-    COGL_PIPELINE_STATE_LAYERS |
-    COGL_PIPELINE_STATE_LIGHTING |
-    COGL_PIPELINE_STATE_BLEND |
-    COGL_PIPELINE_STATE_USER_SHADER,
-
-  COGL_PIPELINE_STATE_NEEDS_BIG_STATE =
-    COGL_PIPELINE_STATE_LIGHTING |
-    COGL_PIPELINE_STATE_ALPHA_FUNC |
-    COGL_PIPELINE_STATE_ALPHA_FUNC_REFERENCE |
-    COGL_PIPELINE_STATE_BLEND |
-    COGL_PIPELINE_STATE_USER_SHADER |
-    COGL_PIPELINE_STATE_DEPTH |
-    COGL_PIPELINE_STATE_FOG |
-    COGL_PIPELINE_STATE_POINT_SIZE
+  COGL_PIPELINE_STATE_REAL_BLEND_ENABLE =
+    1L<<COGL_PIPELINE_STATE_REAL_BLEND_ENABLE_INDEX,
 
 } CoglPipelineState;
 
+/*
+ * Various special masks that tag state-groups in different ways...
+ */
+
+#define COGL_PIPELINE_STATE_ALL \
+  ((1L<<COGL_PIPELINE_STATE_COUNT) - 1)
+
+#define COGL_PIPELINE_STATE_ALL_SPARSE \
+  (COGL_PIPELINE_STATE_ALL \
+   & ~COGL_PIPELINE_STATE_REAL_BLEND_ENABLE)
+
+#define COGL_PIPELINE_STATE_AFFECTS_BLENDING \
+  (COGL_PIPELINE_STATE_COLOR | \
+   COGL_PIPELINE_STATE_BLEND_ENABLE | \
+   COGL_PIPELINE_STATE_LAYERS | \
+   COGL_PIPELINE_STATE_LIGHTING | \
+   COGL_PIPELINE_STATE_BLEND | \
+   COGL_PIPELINE_STATE_USER_SHADER)
+
+#define COGL_PIPELINE_STATE_NEEDS_BIG_STATE \
+  (COGL_PIPELINE_STATE_LIGHTING | \
+   COGL_PIPELINE_STATE_ALPHA_FUNC | \
+   COGL_PIPELINE_STATE_ALPHA_FUNC_REFERENCE | \
+   COGL_PIPELINE_STATE_BLEND | \
+   COGL_PIPELINE_STATE_USER_SHADER | \
+   COGL_PIPELINE_STATE_DEPTH | \
+   COGL_PIPELINE_STATE_FOG | \
+   COGL_PIPELINE_STATE_POINT_SIZE)
+
+#define COGL_PIPELINE_STATE_MULTI_PROPERTY \
+  (COGL_PIPELINE_STATE_LAYERS | \
+   COGL_PIPELINE_STATE_LIGHTING | \
+   COGL_PIPELINE_STATE_BLEND | \
+   COGL_PIPELINE_STATE_DEPTH | \
+   COGL_PIPELINE_STATE_FOG)
+
+#ifdef HAVE_COGL_GLES2
+  /* Under GLES2 the alpha func becomes part of the fragment program
+     so we can't share programs there */
+#define COGL_PIPELINE_STATE_AFFECTS_FRAGMENT_CODEGEN \
+  (COGL_PIPELINE_STATE_LAYERS | \
+   COGL_PIPELINE_STATE_ALPHA_FUNC | \
+   COGL_PIPELINE_STATE_USER_SHADER)
+#else
+#define COGL_PIPELINE_STATE_AFFECTS_FRAGMENT_CODEGEN \
+  (COGL_PIPELINE_STATE_LAYERS | \
+   COGL_PIPELINE_STATE_USER_SHADER)
+#endif
+
+#define COGL_PIPELINE_STATE_AFFECTS_VERTEX_CODEGEN \
+  (COGL_PIPELINE_STATE_LAYERS | \
+   COGL_PIPELINE_STATE_USER_SHADER)
+
 typedef enum
 {
   COGL_PIPELINE_LIGHTING_STATE_PROPERTY_AMBIENT = 1,
@@ -342,7 +536,7 @@ typedef struct
 {
   /* Determines what fragments are discarded based on their alpha */
   CoglPipelineAlphaFunc alpha_func;
-  GLfloat		alpha_func_reference;
+  float		        alpha_func_reference;
 } CoglPipelineAlphaFuncState;
 
 typedef enum _CoglPipelineBlendEnable
@@ -462,7 +656,7 @@ struct _CoglPipeline
 
   /* The fragment processing backends can associate private data with a
    * pipeline. */
-  void		  *backend_privs[COGL_PIPELINE_N_BACKENDS];
+  void		  *fragend_privs[COGL_PIPELINE_N_FRAGENDS];
 
   /* Whenever a pipeline is modified we increment the age. There's no
    * guarantee that it won't wrap but it can nevertheless be a
@@ -521,10 +715,10 @@ struct _CoglPipeline
    * state with the ancestors of other pipelines and those ancestors
    * could currently be associated with different backends.
    *
-   * Each set bit indicates if the correspondong ->backend_privs[]
+   * Each set bit indicates if the corresponding ->fragend_privs[]
    * entry is valid.
    */
-  unsigned int          backend_priv_set_mask:COGL_PIPELINE_N_BACKENDS;
+  unsigned int          fragend_priv_set_mask:COGL_PIPELINE_N_FRAGENDS;
 
   /* Weak pipelines don't count as dependants on their parents which
    * means that the parent pipeline can be modified without
@@ -558,13 +752,12 @@ struct _CoglPipeline
    * glsl, arbfp and fixed. This identifies the backend being used for
    * the pipeline and any private state the backend has associated
    * with the pipeline. */
-  unsigned int          backend:3;
+  unsigned int          fragend:3;
+  unsigned int          vertend:3;
 };
 
-typedef struct _CoglPipelineBackend
+typedef struct _CoglPipelineFragend
 {
-  int (*get_max_texture_units) (void);
-
   gboolean (*start) (CoglPipeline *pipeline,
                      int n_layers,
                      unsigned long pipelines_difference,
@@ -585,8 +778,43 @@ typedef struct _CoglPipelineBackend
                                    CoglPipelineLayerState change);
 
   void (*free_priv) (CoglPipeline *pipeline);
-  void (*free_layer_priv) (CoglPipelineLayer *layer);
-} CoglPipelineBackend;
+} CoglPipelineFragend;
+
+typedef struct _CoglPipelineVertend
+{
+  gboolean (*start) (CoglPipeline *pipeline,
+                     int n_layers,
+                     unsigned long pipelines_difference);
+  gboolean (*add_layer) (CoglPipeline *pipeline,
+                         CoglPipelineLayer *layer,
+                         unsigned long layers_difference);
+  gboolean (*end) (CoglPipeline *pipeline,
+                   unsigned long pipelines_difference);
+
+  void (*pipeline_pre_change_notify) (CoglPipeline *pipeline,
+                                      CoglPipelineState change,
+                                      const CoglColor *new_color);
+  void (*layer_pre_change_notify) (CoglPipeline *owner,
+                                   CoglPipelineLayer *layer,
+                                   CoglPipelineLayerState change);
+} CoglPipelineVertend;
+
+typedef struct
+{
+  void (*end) (CoglPipeline *pipeline,
+               unsigned long pipelines_difference,
+               int n_tex_coord_attribs);
+  void (*pipeline_pre_change_notify) (CoglPipeline *pipeline,
+                                      CoglPipelineState change,
+                                      const CoglColor *new_color);
+  void (*layer_pre_change_notify) (CoglPipeline *owner,
+                                   CoglPipelineLayer *layer,
+                                   CoglPipelineLayerState change);
+  /* This is called after all of the other functions whenever the
+     pipeline is flushed, even if the pipeline hasn't changed since
+     the last flush */
+  void (* pre_paint) (CoglPipeline *pipeline);
+} CoglPipelineProgend;
 
 typedef enum
 {
@@ -595,8 +823,12 @@ typedef enum
   COGL_PIPELINE_PROGRAM_TYPE_FIXED
 } CoglPipelineProgramType;
 
-extern const CoglPipelineBackend *
-_cogl_pipeline_backends[COGL_PIPELINE_N_BACKENDS];
+extern const CoglPipelineFragend *
+_cogl_pipeline_fragends[COGL_PIPELINE_N_FRAGENDS];
+extern const CoglPipelineVertend *
+_cogl_pipeline_vertends[COGL_PIPELINE_N_VERTENDS];
+extern const CoglPipelineProgend *
+_cogl_pipeline_progends[];
 
 void
 _cogl_pipeline_init_default_pipeline (void);
@@ -604,6 +836,23 @@ _cogl_pipeline_init_default_pipeline (void);
 void
 _cogl_pipeline_init_default_layers (void);
 
+static inline CoglPipeline *
+_cogl_pipeline_get_parent (CoglPipeline *pipeline)
+{
+  CoglPipelineNode *parent_node = COGL_PIPELINE_NODE (pipeline)->parent;
+  return COGL_PIPELINE (parent_node);
+}
+
+static inline CoglPipeline *
+_cogl_pipeline_get_authority (CoglPipeline *pipeline,
+                              unsigned long difference)
+{
+  CoglPipeline *authority = pipeline;
+  while (!(authority->differences & difference))
+    authority = _cogl_pipeline_get_parent (authority);
+  return authority;
+}
+
 /*
  * SECTION:cogl-pipeline-internals
  * @short_description: Functions for creating custom primitives that make use
@@ -684,10 +933,13 @@ _cogl_get_max_texture_image_units (void);
 
 
 void
-_cogl_use_program (GLuint gl_program, CoglPipelineProgramType type);
+_cogl_use_fragment_program (GLuint gl_program, CoglPipelineProgramType type);
+
+void
+_cogl_use_vertex_program (GLuint gl_program, CoglPipelineProgramType type);
 
 unsigned int
-_cogl_get_n_args_for_combine_func (GLint func);
+_cogl_get_n_args_for_combine_func (CoglPipelineCombineFunc func);
 
 /*
  * _cogl_pipeline_weak_copy:
@@ -813,7 +1065,10 @@ _cogl_pipeline_weak_copy (CoglPipeline *pipeline,
                           void *user_data);
 
 void
-_cogl_pipeline_set_backend (CoglPipeline *pipeline, int backend);
+_cogl_pipeline_set_fragend (CoglPipeline *pipeline, int fragend);
+
+void
+_cogl_pipeline_set_vertend (CoglPipeline *pipeline, int vertend);
 
 CoglPipeline *
 _cogl_pipeline_get_parent (CoglPipeline *pipeline);
@@ -822,14 +1077,34 @@ void
 _cogl_pipeline_get_colorubv (CoglPipeline *pipeline,
                              guint8       *color);
 
+/* XXX: At some point it could be good for this to accept a mask of
+ * the state groups we are interested in comparing since we can
+ * probably use that information in a number situations to reduce
+ * the work we do. */
 unsigned long
 _cogl_pipeline_compare_differences (CoglPipeline *pipeline0,
                                     CoglPipeline *pipeline1);
 
+/* Sometimes when evaluating pipelines, either during comparisons or
+ * if calculating a hash value we need to tweak the evaluation
+ * semantics */
+typedef enum _CoglPipelineEvalFlags
+{
+  COGL_PIPELINE_EVAL_FLAG_NONE = 0
+} CoglPipelineEvalFlags;
+
 gboolean
 _cogl_pipeline_equal (CoglPipeline *pipeline0,
                       CoglPipeline *pipeline1,
-                      gboolean skip_gl_color);
+                      unsigned long differences,
+                      unsigned long layer_differences,
+                      CoglPipelineEvalFlags flags);
+
+unsigned int
+_cogl_pipeline_hash (CoglPipeline *pipeline,
+                     unsigned long differences,
+                     unsigned long layer_differences,
+                     CoglPipelineEvalFlags flags);
 
 CoglPipeline *
 _cogl_pipeline_journal_ref (CoglPipeline *pipeline);
@@ -845,6 +1120,10 @@ CoglPipelineFilter
 _cogl_pipeline_get_layer_mag_filter (CoglPipeline *pipeline,
                                      int layer_index);
 
+const CoglMatrix *
+_cogl_pipeline_get_layer_matrix (CoglPipeline *pipeline,
+                                 int layer_index);
+
 void
 _cogl_pipeline_texture_storage_change_notify (CoglHandle texture);
 
@@ -873,6 +1152,11 @@ CoglPipeline *
 _cogl_pipeline_get_authority (CoglPipeline *pipeline,
                               unsigned long difference);
 
+CoglPipeline *
+_cogl_pipeline_find_equivalent_parent (CoglPipeline *pipeline,
+                                       CoglPipelineState pipeline_state,
+                                       CoglPipelineLayerState layer_state);
+
 CoglHandle
 _cogl_pipeline_get_layer_texture (CoglPipeline *pipeline,
                                   int layer_index);
@@ -920,6 +1204,9 @@ _cogl_pipeline_layer_get_type (CoglPipelineLayer *layer);
 CoglHandle
 _cogl_pipeline_layer_get_texture (CoglPipelineLayer *layer);
 
+CoglHandle
+_cogl_pipeline_layer_get_texture_real (CoglPipelineLayer *layer);
+
 CoglPipelineFilter
 _cogl_pipeline_layer_get_min_filter (CoglPipelineLayer *layer);
 
@@ -961,9 +1248,11 @@ gboolean
 _cogl_pipeline_need_texture_combine_separate
                                     (CoglPipelineLayer *combine_authority);
 
-CoglPipeline *
-_cogl_pipeline_find_codegen_authority (CoglPipeline *pipeline,
-                                       CoglHandle user_program);
+void
+_cogl_pipeline_init_state_hash_functions (void);
+
+void
+_cogl_pipeline_init_layer_state_hash_functions (void);
 
 #endif /* __COGL_PIPELINE_PRIVATE_H */
 
