@@ -48,7 +48,6 @@
 #include "clutter-private.h"
 #include "clutter-main.h"
 #include "clutter-stage-private.h"
-#include "clutter-version.h"
 
 static ClutterBackendEGL *backend_singleton = NULL;
 
@@ -505,21 +504,6 @@ clutter_backend_egl_ensure_context (ClutterBackend *backend,
 #endif /* COGL_HAS_XLIB_SUPPORT */
 }
 
-static void
-clutter_backend_egl_redraw (ClutterBackend *backend,
-                            ClutterStage   *stage)
-{
-  ClutterStageWindow *impl;
-
-  impl = _clutter_stage_get_window (stage);
-  if (!impl)
-    return;
-
-  g_assert (CLUTTER_IS_STAGE_EGL (impl));
-
-  _clutter_stage_egl_redraw (CLUTTER_STAGE_EGL (impl), stage);
-}
-
 #ifndef COGL_HAS_XLIB_SUPPORT
 static ClutterDeviceManager *
 clutter_backend_egl_get_device_manager (ClutterBackend *backend)
@@ -759,9 +743,6 @@ clutter_backend_egl_create_stage (ClutterBackend  *backend,
   ClutterStageWindow *stage;
   ClutterStageX11 *stage_x11;
 
-  CLUTTER_NOTE (BACKEND, "Creating stage of type '%s'",
-                g_type_name (CLUTTER_STAGE_TYPE));
-
   stage = g_object_new (CLUTTER_TYPE_STAGE_EGL, NULL);
 
   /* copy backend data into the stage */
@@ -770,7 +751,7 @@ clutter_backend_egl_create_stage (ClutterBackend  *backend,
   stage_x11->backend = backend_x11;
 
   translator = CLUTTER_EVENT_TRANSLATOR (stage_x11);
-  _clutter_backend_x11_add_event_translator (backend_x11, translator);
+  _clutter_backend_add_event_translator (backend, translator);
 
   CLUTTER_NOTE (MISC, "EGLX stage created (display:%p, screen:%d, root:%u)",
                 backend_x11->xdpy,
@@ -879,7 +860,6 @@ _clutter_backend_egl_class_init (ClutterBackendEGLClass *klass)
   backend_class->create_stage       = clutter_backend_egl_create_stage;
   backend_class->create_context     = clutter_backend_egl_create_context;
   backend_class->ensure_context     = clutter_backend_egl_ensure_context;
-  backend_class->redraw             = clutter_backend_egl_redraw;
 
 #ifdef COGL_HAS_XLIB_SUPPORT
   backendx11_class->get_visual_info = clutter_backend_egl_get_visual_info;
