@@ -71,10 +71,9 @@ G_DEFINE_TYPE_WITH_CODE (ClutterStageGLX,
 static void
 clutter_stage_glx_unrealize (ClutterStageWindow *stage_window)
 {
-  ClutterBackend *backend = clutter_get_default_backend ();
-  ClutterBackendX11 *backend_x11 = CLUTTER_BACKEND_X11 (backend);
   ClutterStageX11 *stage_x11 = CLUTTER_STAGE_X11 (stage_window);
   ClutterStageGLX *stage_glx = CLUTTER_STAGE_GLX (stage_window);
+  ClutterBackendX11 *backend_x11 = stage_x11->backend;
 
   /* Note unrealize should free up any backend stage related resources */
   CLUTTER_NOTE (BACKEND, "Unrealizing GLX stage [%p]", stage_glx);
@@ -101,19 +100,17 @@ clutter_stage_glx_realize (ClutterStageWindow *stage_window)
   ClutterStageGLX *stage_glx = CLUTTER_STAGE_GLX (stage_window);
   ClutterBackendX11 *backend_x11;
   ClutterBackendGLX *backend_glx;
-  ClutterBackend *backend;
 
   CLUTTER_NOTE (ACTOR, "Realizing stage '%s' [%p]",
                 G_OBJECT_TYPE_NAME (stage_window),
                 stage_window);
 
-  backend     = clutter_get_default_backend ();
-  backend_glx = CLUTTER_BACKEND_GLX (backend);
-  backend_x11 = CLUTTER_BACKEND_X11 (backend);
-
   if (!_clutter_stage_x11_create_window (stage_x11))
     return FALSE;
 
+  backend_x11 = stage_x11->backend;
+  backend_glx = CLUTTER_BACKEND_GLX (backend_x11);
+
   if (stage_glx->glxwin == None)
     {
       int major;
@@ -220,14 +217,12 @@ clutter_stage_glx_ignoring_redraw_clips (ClutterStageWindow *stage_window)
  * - when we come to redraw; if the bounding box is smaller than the
  *   stage we scissor the redraw to that box and use
  *   GLX_MESA_copy_sub_buffer to present the redraw to the front
- *   buffer. Some heuristics are used to decide when a clipped redraw
- *   should be promoted into a full stage redraw.
+ *   buffer.
  *
- * Currently we simply check that the bounding box height is < 300
- * pixels.
- *
- * XXX: we don't have any empirical data telling us what a sensible
- * thresholds is!
+ * XXX - In theory, we should have some sort of heuristics to promote
+ * a clipped redraw to a full screen redraw; in reality, it turns out
+ * that promotion is fairly expensive. See the Clutter bug described
+ * at: http://bugzilla.clutter-project.org/show_bug.cgi?id=2136 .
  *
  * TODO - we should use different heuristics depending on whether the
  * framebuffer is on screen and not redirected by a compositor VS
@@ -251,7 +246,6 @@ clutter_stage_glx_ignoring_redraw_clips (ClutterStageWindow *stage_window)
  * By promoting to a full stage redraw we trade off the cost involved
  * in rasterizing the extra pixels vs avoiding to use a blit to
  * present the back buffer.
- *
  */
 static void
 clutter_stage_glx_add_redraw_clip (ClutterStageWindow *stage_window,
@@ -286,7 +280,8 @@ clutter_stage_glx_add_redraw_clip (ClutterStageWindow *stage_window,
     }
   else if (stage_glx->bounding_redraw_clip.width > 0)
     {
-      clutter_geometry_union (&stage_glx->bounding_redraw_clip, stage_clip,
+      clutter_geometry_union (&stage_glx->bounding_redraw_clip,
+                              stage_clip,
 			      &stage_glx->bounding_redraw_clip);
     }
 
@@ -299,6 +294,11 @@ clutter_stage_glx_add_redraw_clip (ClutterStageWindow *stage_window,
    * expensive than redrawing the additional 30% to avoid the blit.
    *
    * FIXME: This threshold was plucked out of thin air!
+   *
+   * The threshold has been disabled after verifying that it indeed
+   * made redraws more expensive than intended; see bug reference:
+   *
+   * http://bugzilla.clutter-project.org/show_bug.cgi?id=2136
    */
   if (redraw_area > (stage_area * 0.7f))
     {
@@ -311,71 +311,6 @@ clutter_stage_glx_add_redraw_clip (ClutterStageWindow *stage_window,
   stage_glx->initialized_redraw_clip = TRUE;
 }
 
-static void
-clutter_stage_window_iface_init (ClutterStageWindowIface *iface)
-{
-  clutter_stage_window_parent_iface = g_type_interface_peek_parent (iface);
-
-  iface->realize = clutter_stage_glx_realize;
-  iface->unrealize = clutter_stage_glx_unrealize;
-  iface->get_pending_swaps = clutter_stage_glx_get_pending_swaps;
-
-  iface->add_redraw_clip = clutter_stage_glx_add_redraw_clip;
-  iface->has_redraw_clips = clutter_stage_glx_has_redraw_clips;
-  iface->ignoring_redraw_clips = clutter_stage_glx_ignoring_redraw_clips;
-
-  /* the rest is inherited from ClutterStageX11 */
-}
-
-static ClutterTranslateReturn
-clutter_stage_glx_translate_event (ClutterEventTranslator *translator,
-                                   gpointer                native,
-                                   ClutterEvent           *event)
-{
-#ifdef GLX_INTEL_swap_event
-  ClutterBackendGLX *backend_glx;
-  XEvent *xevent = native;
-
-  backend_glx = CLUTTER_BACKEND_GLX (clutter_get_default_backend ());
-
-  if (xevent->type == (backend_glx->event_base + GLX_BufferSwapComplete))
-    {
-      ClutterStageX11 *stage_x11 = CLUTTER_STAGE_X11 (translator);
-      ClutterStageGLX *stage_glx = CLUTTER_STAGE_GLX (translator);
-      GLXBufferSwapComplete *swap_complete_event;
-
-      swap_complete_event = (GLXBufferSwapComplete *) xevent;
-
-      if (stage_x11->xwin == swap_complete_event->drawable)
-        {
-	  /* Early versions of the swap_event implementation in Mesa
-	   * deliver BufferSwapComplete event when not selected for,
-	   * so if we get a swap event we aren't expecting, just ignore it.
-	   *
-	   * https://bugs.freedesktop.org/show_bug.cgi?id=27962
-	   */
-          if (stage_glx->pending_swaps > 0)
-            stage_glx->pending_swaps--;
-
-          return CLUTTER_TRANSLATE_REMOVE;
-        }
-    }
-#endif
-
-  /* chain up to the common X11 implementation */
-  return clutter_event_translator_parent_iface->translate_event (translator,
-                                                                 native,
-                                                                 event);
-}
-
-static void
-clutter_event_translator_iface_init (ClutterEventTranslatorIface *iface)
-{
-  clutter_event_translator_parent_iface = g_type_interface_peek_parent (iface);
-
-  iface->translate_event = clutter_stage_glx_translate_event;
-}
-
 #ifdef HAVE_DRM
 static int
 drm_wait_vblank(int fd, drm_wait_vblank_t *vbl)
@@ -426,18 +361,18 @@ wait_for_vblank (ClutterBackendGLX *backend_glx)
     }
 }
 
-void
-_clutter_stage_glx_redraw (ClutterStageGLX *stage_glx,
-                           ClutterStage    *stage)
+static void
+clutter_stage_glx_redraw (ClutterStageWindow *stage_window)
 {
-  ClutterBackend    *backend;
   ClutterBackendX11 *backend_x11;
   ClutterBackendGLX *backend_glx;
-  ClutterStageX11   *stage_x11;
-  GLXDrawable        drawable;
-  unsigned int       video_sync_count;
-  gboolean           may_use_clipped_redraw;
-  gboolean           use_clipped_redraw;
+  ClutterStageX11 *stage_x11;
+  ClutterStageGLX *stage_glx;
+  GLXDrawable drawable;
+  unsigned int video_sync_count;
+  gboolean may_use_clipped_redraw;
+  gboolean use_clipped_redraw;
+
   CLUTTER_STATIC_TIMER (painting_timer,
                         "Redrawing", /* parent */
                         "Painting actors",
@@ -454,13 +389,14 @@ _clutter_stage_glx_redraw (ClutterStageGLX *stage_glx,
                         "The time spent in _glx_blit_sub_buffer",
                         0 /* no application private data */);
 
-  stage_x11 = CLUTTER_STAGE_X11 (stage_glx);
+  stage_x11 = CLUTTER_STAGE_X11 (stage_window);
   if (stage_x11->xwin == None)
     return;
 
-  backend     = clutter_get_default_backend ();
-  backend_x11 = CLUTTER_BACKEND_X11 (backend);
-  backend_glx = CLUTTER_BACKEND_GLX (backend);
+  stage_glx = CLUTTER_STAGE_GLX (stage_window);
+
+  backend_x11 = stage_x11->backend;
+  backend_glx = CLUTTER_BACKEND_GLX (backend_x11);
 
   CLUTTER_TIMER_START (_clutter_uprof_context, painting_timer);
 
@@ -493,17 +429,19 @@ _clutter_stage_glx_redraw (ClutterStageGLX *stage_glx,
                                        stage_glx->bounding_redraw_clip.y,
                                        stage_glx->bounding_redraw_clip.width,
                                        stage_glx->bounding_redraw_clip.height);
-      _clutter_stage_do_paint (stage, &stage_glx->bounding_redraw_clip);
+      _clutter_stage_do_paint (stage_x11->wrapper,
+                               &stage_glx->bounding_redraw_clip);
       cogl_clip_pop ();
     }
   else
-    _clutter_stage_do_paint (stage, NULL);
+    _clutter_stage_do_paint (stage_x11->wrapper, NULL);
 
-  if ((clutter_paint_debug_flags & CLUTTER_DEBUG_REDRAWS) &&
-      may_use_clipped_redraw)
+  if (may_use_clipped_redraw &&
+      G_UNLIKELY ((clutter_paint_debug_flags & CLUTTER_DEBUG_REDRAWS)))
     {
-      ClutterGeometry *clip = &stage_glx->bounding_redraw_clip;
       static CoglMaterial *outline = NULL;
+      ClutterGeometry *clip = &stage_glx->bounding_redraw_clip;
+      ClutterActor *actor = CLUTTER_ACTOR (stage_x11->wrapper);
       CoglHandle vbo;
       float x_1 = clip->x;
       float x_2 = clip->x + clip->width;
@@ -535,8 +473,7 @@ _clutter_stage_glx_redraw (ClutterStageGLX *stage_glx,
 
       cogl_push_matrix ();
       cogl_matrix_init_identity (&modelview);
-      _clutter_actor_apply_modelview_transform (CLUTTER_ACTOR (stage),
-                                                &modelview);
+      _clutter_actor_apply_modelview_transform (actor, &modelview);
       cogl_set_modelview_matrix (&modelview);
       cogl_set_source (outline);
       cogl_vertex_buffer_draw (vbo, COGL_VERTICES_MODE_LINE_LOOP,
@@ -568,6 +505,7 @@ _clutter_stage_glx_redraw (ClutterStageGLX *stage_glx,
     {
       ClutterGeometry *clip = &stage_glx->bounding_redraw_clip;
       ClutterGeometry copy_area;
+      ClutterActor *actor;
 
       CLUTTER_NOTE (BACKEND,
                     "_glx_blit_sub_buffer (window: 0x%lx, "
@@ -586,7 +524,8 @@ _clutter_stage_glx_redraw (ClutterStageGLX *stage_glx,
        * in this case a full redraw should be queued by the resize
        * anyway so it should only exhibit temporary artefacts.
        */
-      copy_area.y = clutter_actor_get_height (CLUTTER_ACTOR (stage))
+      actor = CLUTTER_ACTOR (stage_x11->wrapper);
+      copy_area.y = clutter_actor_get_height (actor)
                   - clip->y
                   - clip->height;
       copy_area.x = clip->x;
@@ -616,7 +555,7 @@ _clutter_stage_glx_redraw (ClutterStageGLX *stage_glx,
            * See where we call glXSwapBuffers for more details.
            */
           glFinish ();
-          wait_for_vblank (CLUTTER_BACKEND_GLX (backend));
+          wait_for_vblank (backend_glx);
         }
       else if (backend_glx->get_video_sync)
         {
@@ -625,10 +564,10 @@ _clutter_stage_glx_redraw (ClutterStageGLX *stage_glx,
            * any waits if we can see that the video sync count has
            * already progressed. */
           if (backend_glx->last_video_sync_count == video_sync_count)
-            wait_for_vblank (CLUTTER_BACKEND_GLX (backend));
+            wait_for_vblank (backend_glx);
         }
       else
-        wait_for_vblank (CLUTTER_BACKEND_GLX (backend));
+        wait_for_vblank (backend_glx);
 
       CLUTTER_TIMER_START (_clutter_uprof_context, blit_sub_buffer_timer);
       _clutter_backend_glx_blit_sub_buffer (backend_glx,
@@ -672,12 +611,13 @@ _clutter_stage_glx_redraw (ClutterStageGLX *stage_glx,
            */
           glFinish ();
 
-          wait_for_vblank (CLUTTER_BACKEND_GLX (backend));
+          wait_for_vblank (backend_glx);
         }
 
       CLUTTER_TIMER_START (_clutter_uprof_context, swapbuffers_timer);
       glXSwapBuffers (backend_x11->xdpy, drawable);
       CLUTTER_TIMER_STOP (_clutter_uprof_context, swapbuffers_timer);
+
       _cogl_swap_buffers_notify ();
     }
 
@@ -688,3 +628,69 @@ _clutter_stage_glx_redraw (ClutterStageGLX *stage_glx,
 
   stage_glx->frame_count++;
 }
+
+static void
+clutter_stage_window_iface_init (ClutterStageWindowIface *iface)
+{
+  clutter_stage_window_parent_iface = g_type_interface_peek_parent (iface);
+
+  iface->realize = clutter_stage_glx_realize;
+  iface->unrealize = clutter_stage_glx_unrealize;
+  iface->get_pending_swaps = clutter_stage_glx_get_pending_swaps;
+
+  iface->add_redraw_clip = clutter_stage_glx_add_redraw_clip;
+  iface->has_redraw_clips = clutter_stage_glx_has_redraw_clips;
+  iface->ignoring_redraw_clips = clutter_stage_glx_ignoring_redraw_clips;
+  iface->redraw = clutter_stage_glx_redraw;
+
+  /* the rest is inherited from ClutterStageX11 */
+}
+
+static ClutterTranslateReturn
+clutter_stage_glx_translate_event (ClutterEventTranslator *translator,
+                                   gpointer                native,
+                                   ClutterEvent           *event)
+{
+#ifdef GLX_INTEL_swap_event
+  ClutterBackendGLX *backend_glx;
+  XEvent *xevent = native;
+
+  backend_glx = CLUTTER_BACKEND_GLX (clutter_get_default_backend ());
+
+  if (xevent->type == (backend_glx->event_base + GLX_BufferSwapComplete))
+    {
+      ClutterStageX11 *stage_x11 = CLUTTER_STAGE_X11 (translator);
+      ClutterStageGLX *stage_glx = CLUTTER_STAGE_GLX (translator);
+      GLXBufferSwapComplete *swap_complete_event;
+
+      swap_complete_event = (GLXBufferSwapComplete *) xevent;
+
+      if (stage_x11->xwin == swap_complete_event->drawable)
+        {
+	  /* Early versions of the swap_event implementation in Mesa
+	   * deliver BufferSwapComplete event when not selected for,
+	   * so if we get a swap event we aren't expecting, just ignore it.
+	   *
+	   * https://bugs.freedesktop.org/show_bug.cgi?id=27962
+	   */
+          if (stage_glx->pending_swaps > 0)
+            stage_glx->pending_swaps--;
+
+          return CLUTTER_TRANSLATE_REMOVE;
+        }
+    }
+#endif
+
+  /* chain up to the common X11 implementation */
+  return clutter_event_translator_parent_iface->translate_event (translator,
+                                                                 native,
+                                                                 event);
+}
+
+static void
+clutter_event_translator_iface_init (ClutterEventTranslatorIface *iface)
+{
+  clutter_event_translator_parent_iface = g_type_interface_peek_parent (iface);
+
+  iface->translate_event = clutter_stage_glx_translate_event;
+}
